@@ -5,7 +5,10 @@ from PyQt5.QtWidgets import QWidget, QPushButton, QHBoxLayout, QStackedWidget
 
 from feeluown.gui.components import Avatar
 from feeluown.gui.widgets import (
-    LeftArrowButton, RightArrowButton, SearchButton, SettingsButton,
+    LeftArrowButton,
+    RightArrowButton,
+    SearchSwitchButton,
+    SettingsButton,
 )
 from feeluown.gui.widgets.magicbox import MagicBox
 from feeluown.gui.widgets.statusline import StatusLine
@@ -13,17 +16,18 @@ from feeluown.gui.widgets.statusline import StatusLine
 if TYPE_CHECKING:
     from feeluown.app.gui_app import GuiApp
 
-
 ButtonSize = (30, 30)
 ButtonSpacing = int(ButtonSize[0] / 6)
 
 
 class ToolbarButton(QPushButton):
+
     def __init__(self, text, parent=None):
         super().__init__(text, parent=parent)
 
 
 class BottomPanel(QWidget):
+
     def __init__(self, app: 'GuiApp', parent=None):
         super().__init__(parent)
         self._app = app
@@ -34,7 +38,9 @@ class BottomPanel(QWidget):
 
         self.magicbox = MagicBox(self._app)
 
-        self._stack_switch = SearchButton(length=ButtonSize[0])
+        self._stack_switch = SearchSwitchButton(length=ButtonSize[0])
+        self._stack_switch.setToolTip("显示搜索框")
+        self._stack_switch.setCheckable(True)
         self._stacked_widget = QStackedWidget(self)
         self._stacked_widget.addWidget(self.magicbox)
         self._stack_switch.hide()
@@ -50,6 +56,8 @@ class BottomPanel(QWidget):
         self.back_btn.clicked.connect(self._app.browser.back)
         self.forward_btn.clicked.connect(self._app.browser.forward)
         self._stack_switch.clicked.connect(self._show_next_stacked_widget)
+        self._stack_switch.toggled.connect(self._on_stack_switch_toggled)
+        self._stacked_widget.currentChanged.connect(self._on_stacked_widget_changed)
 
         self._setup_ui()
 
@@ -84,6 +92,19 @@ class BottomPanel(QWidget):
         else:
             next_index = 0
         self._stacked_widget.setCurrentIndex(next_index)
+        if self._stacked_widget.currentWidget() == self.magicbox:
+            self.magicbox.setFocus()
+
+    def _on_stack_switch_toggled(self, checked):
+        if checked:
+            self._stack_switch.setToolTip("关闭搜索框")
+        else:
+            self._stack_switch.setToolTip("显示搜索框")
+
+    def _on_stacked_widget_changed(self, _):
+        self._stack_switch.setChecked(
+            self._stacked_widget.currentWidget() == self.magicbox
+        )
 
     def add_stacked_widget(self, widget):
         """
