@@ -1,47 +1,16 @@
 from typing import runtime_checkable, Protocol, List, Tuple, Optional, Dict
 from abc import abstractmethod
-
 from feeluown.media import Quality, Media
+from feeluown.excs import NoUserLoggedIn
 from .models import (
     BriefCommentModel, SongModel, VideoModel, AlbumModel, ArtistModel,
-    PlaylistModel, UserModel, ModelType,
-)
-from .model_protocol import (
-    BriefArtistProtocol, BriefSongProtocol, SongProtocol,
-    BriefVideoProtocol, VideoProtocol,
-    LyricProtocol,
+    PlaylistModel, UserModel, ModelType, BriefArtistModel, BriefSongModel,
+    LyricModel, BriefVideoModel,
 )
+
 from .flags import Flags as PF
 
 
-__all__ = (
-    'SupportsAlbumGet',
-    'SupportsAlbumSongsReader',
-
-    'SupportsArtistAlbumsReader',
-    'SupportsArtistGet',
-    'SupportsArtistSongsReader',
-
-    'SupportsCurrentUser',
-
-    'SupportsPlaylistAddSong',
-    'SupportsPlaylistGet',
-    'SupportsPlaylistRemoveSong',
-    'SupportsPlaylistSongsReader',
-
-    'SupportsSongGet',
-    'SupportsSongHotComments',
-    'SupportsSongLyric',
-    'SupportsSongMV',
-    'SupportsSongMultiQuality',
-    'SupportsSongSimilar',
-    'SupportsSongWebUrl',
-
-    'SupportsVideoGet',
-    'SupportsVideoMultiQuality',
-)
-
-
 ID = str
 _FlagProtocolMapping: Dict[Tuple[ModelType, PF], type] = {}
 
@@ -90,7 +59,7 @@ class SupportsSongGet(Protocol):
 @runtime_checkable
 class SupportsSongSimilar(Protocol):
     @abstractmethod
-    def song_list_similar(self, song: BriefSongProtocol) -> List[BriefSongProtocol]:
+    def song_list_similar(self, song: BriefSongModel) -> List[BriefSongModel]:
         """List similar songs
         """
         raise NotImplementedError
@@ -100,7 +69,7 @@ class SupportsSongSimilar(Protocol):
 @runtime_checkable
 class SupportsSongMultiQuality(Protocol):
     @abstractmethod
-    def song_list_quality(self, song: BriefSongProtocol) -> List[Quality.Audio]:
+    def song_list_quality(self, song: BriefSongModel) -> List[Quality.Audio]:
         """List all possible qualities
 
         Please ensure all the qualities are valid. `song_get_media(song, quality)`
@@ -110,7 +79,7 @@ class SupportsSongMultiQuality(Protocol):
 
     @abstractmethod
     def song_select_media(
-        self, song: BriefSongProtocol, policy=None
+        self, song: BriefSongModel, policy=None
     ) -> Tuple[Media, Quality.Audio]:
         """Select a media by the quality sorting policy
 
@@ -120,7 +89,7 @@ class SupportsSongMultiQuality(Protocol):
 
     @abstractmethod
     def song_get_media(
-        self, song: BriefVideoProtocol, quality: Quality.Audio
+        self, song: BriefVideoModel, quality: Quality.Audio
     ) -> Optional[Media]:
         """Get song's media by a specified quality
 
@@ -132,21 +101,21 @@ class SupportsSongMultiQuality(Protocol):
 @eq(ModelType.song, PF.hot_comments)
 @runtime_checkable
 class SupportsSongHotComments(Protocol):
-    def song_list_hot_comments(self, song: BriefSongProtocol) -> List[BriefCommentModel]:
+    def song_list_hot_comments(self, song: BriefSongModel) -> List[BriefCommentModel]:
         raise NotImplementedError
 
 
 @eq(ModelType.song, PF.web_url)
 @runtime_checkable
 class SupportsSongWebUrl(Protocol):
-    def song_get_web_url(self, song: BriefSongProtocol) -> str:
+    def song_get_web_url(self, song: BriefSongModel) -> str:
         raise NotImplementedError
 
 
 @eq(ModelType.song, PF.lyric)
 @runtime_checkable
 class SupportsSongLyric(Protocol):
-    def song_get_lyric(self, song: BriefSongProtocol) -> Optional[LyricProtocol]:
+    def song_get_lyric(self, song: BriefSongModel) -> Optional[LyricModel]:
         """Get music video of the song
         """
         raise NotImplementedError
@@ -155,7 +124,7 @@ class SupportsSongLyric(Protocol):
 @eq(ModelType.song, PF.mv)
 @runtime_checkable
 class SupportsSongMV(Protocol):
-    def song_get_mv(self, song: BriefSongProtocol) -> Optional[VideoProtocol]:
+    def song_get_mv(self, song: BriefSongModel) -> Optional[VideoModel]:
         """Get music video of the song
 
         """
@@ -182,7 +151,7 @@ class SupportsAlbumGet(Protocol):
 @runtime_checkable
 class SupportsAlbumSongsReader(Protocol):
     @abstractmethod
-    def album_create_songs_rd(self, album) -> List[SongProtocol]:
+    def album_create_songs_rd(self, album) -> List[SongModel]:
         raise NotImplementedError
 
 
@@ -206,7 +175,7 @@ class SupportsArtistGet(Protocol):
 @runtime_checkable
 class SupportsArtistSongsReader(Protocol):
     @abstractmethod
-    def artist_create_songs_rd(self, artist: BriefArtistProtocol):
+    def artist_create_songs_rd(self, artist: BriefArtistModel):
         """Create songs reader of the artist
         """
         raise NotImplementedError
@@ -216,7 +185,7 @@ class SupportsArtistSongsReader(Protocol):
 @runtime_checkable
 class SupportsArtistAlbumsReader(Protocol):
     @abstractmethod
-    def artist_create_albums_rd(self, artist: BriefArtistProtocol):
+    def artist_create_albums_rd(self, artist: BriefArtistModel):
         """Create albums reader of the artist
         """
         raise NotImplementedError
@@ -225,7 +194,7 @@ class SupportsArtistAlbumsReader(Protocol):
 @runtime_checkable
 class SupportsArtistContributedAlbumsReader(Protocol):
     @abstractmethod
-    def artist_create_contributed_albums_rd(self, artist: BriefArtistProtocol):
+    def artist_create_contributed_albums_rd(self, artist: BriefArtistModel):
         """Create contributed albums reader of the artist
         """
         raise NotImplementedError
@@ -251,16 +220,16 @@ class SupportsVideoGet(Protocol):
 @runtime_checkable
 class SupportsVideoMultiQuality(Protocol):
     @abstractmethod
-    def video_list_quality(self, video: BriefVideoProtocol) -> List[Quality.Video]:
+    def video_list_quality(self, video: BriefVideoModel) -> List[Quality.Video]:
         raise NotImplementedError
 
     @abstractmethod
     def video_select_media(
-            self, video: BriefVideoProtocol, policy=None) -> Tuple[Media, Quality.Video]:
+            self, video: BriefVideoModel, policy=None) -> Tuple[Media, Quality.Video]:
         raise NotImplementedError
 
     @abstractmethod
-    def video_get_media(self, video: BriefVideoProtocol, quality) -> Optional[Media]:
+    def video_get_media(self, video: BriefVideoModel, quality) -> Optional[Media]:
         raise NotImplementedError
 
 
@@ -280,6 +249,28 @@ class SupportsPlaylistGet(Protocol):
         raise NotImplementedError
 
 
+@runtime_checkable
+class SupportsPlaylistCreateByName(Protocol):
+    @abstractmethod
+    def playlist_create_by_name(self, name) -> PlaylistModel:
+        """Create playlist for user logged in.
+
+        :raises NoUserLoggedIn:
+        :raises ProviderIOError:
+        """
+
+
+@runtime_checkable
+class SupportsPlaylistDelete(Protocol):
+    @abstractmethod
+    def playlist_delete(self, identifier: ID) -> bool:
+        """
+        :raises ModelNotFound: model not found by the identifier
+        :raises ProviderIOError:
+        """
+        raise NotImplementedError
+
+
 @eq(ModelType.playlist, PF.songs_rd)
 @runtime_checkable
 class SupportsPlaylistSongsReader(Protocol):
@@ -319,4 +310,89 @@ class SupportsCurrentUser(Protocol):
         """Get current logged in user
 
         :raises NoUserLoggedIn: there is no logged in user.
+
+        .. versionchanged: 4.0
+            It can return Optional[UserModel], NoUserLoggedIn makes no sense.
         """
+
+    def get_current_user_or_none(self) -> Optional[UserModel]:
+        """
+        .. versionadded: 4.0
+        """
+        try:
+            return self.get_current_user()
+        except NoUserLoggedIn:
+            return None
+
+
+@runtime_checkable
+class SupportsCurrentUserListPlaylists(Protocol):
+    @abstractmethod
+    def current_user_list_playlists(self):
+        """
+        : raises NoUserLoggedIn:
+        """
+
+
+#
+# Protocols for current user favorites/collections
+#
+@runtime_checkable
+class SupportsCurrentUserFavSongsReader(Protocol):
+    @abstractmethod
+    def current_user_fav_create_songs_rd(self):
+        """
+        : raises NoUserLoggedIn:
+        """
+
+
+@runtime_checkable
+class SupportsCurrentUserFavAlbumsReader(Protocol):
+    @abstractmethod
+    def current_user_fav_create_albums_rd(self):
+        pass
+
+
+@runtime_checkable
+class SupportsCurrentUserFavArtistsReader(Protocol):
+    @abstractmethod
+    def current_user_fav_create_artists_rd(self):
+        pass
+
+
+@runtime_checkable
+class SupportsCurrentUserFavPlaylistsReader(Protocol):
+    @abstractmethod
+    def current_user_fav_create_playlists_rd(self):
+        pass
+
+
+@runtime_checkable
+class SupportsCurrentUserFavVideosReader(Protocol):
+    @abstractmethod
+    def current_user_fav_create_videos_rd(self):
+        pass
+
+
+#
+# Protocols for recommendation.
+#
+@runtime_checkable
+class SupportsRecListDailySongs(Protocol):
+    @abstractmethod
+    def rec_list_daily_songs(self) -> List[SongModel]:
+        pass
+
+
+@runtime_checkable
+class SupportsRecListDailyPlaylists(Protocol):
+    @abstractmethod
+    def rec_list_daily_playlists(self) -> List[PlaylistModel]:
+        pass
+
+
+@runtime_checkable
+class SupportsRecListDailyAlbums(Protocol):
+    @abstractmethod
+    def rec_list_daily_albums(self) -> List[AlbumModel]:
+        pass
